﻿using Hotline.Application.Contracts.Configurations;
using Hotline.Caches;
using Hotline.CallCenter.Tels;
using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Dtos.Menu;
using Hotline.Share.Dtos.Role;
using Hotline.Share.Dtos.User;
using Hotline.Share.Requests;
using Hotline.Users;
using Identity.Admin.HttpClient;
using Identity.Shared.Dtos;
using Identity.Shared.Dtos.Account;
using Identity.Shared.Dtos.Identity;
using Identity.Shared.Dtos.Role;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Utility.UnifyResponse;

namespace Hotline.Api.Controllers
{
    /// <summary>
    ///  系统管理相关接口
    /// </summary>
    public class SysController : BaseController
    {
        private readonly ISessionContext _sessionContext;
        private readonly IUserRepository _userRepository;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IIdentityClient _identityClient;
        private readonly IOptionsSnapshot<IdentityConfigs> _identityConfigs;
        private readonly IMapper _mapper;
        private readonly ISystemSettingRepository _systemSettingsRepository;
        private readonly ISystemSettingGroupRepository _systemSettingGroupRepository;
        private readonly ITelRepository _telRepository;
        private readonly ISystemMenuRepository _systemMenuRepository;
        private readonly ISystemButtonRepository _systemButtonRepository;
        private readonly ISystemAuthorityRepository _systemAuthorityRepository;
        private readonly ISystemDataAuthorityRepository _systemDataAuthorityRepository;

        /// <summary>
        /// 系统管理相关接口
        /// </summary>
        /// <param name="sessionContext"></param>
        /// <param name="userRepository"></param>
        /// <param name="userCacheManager"></param>
        /// <param name="identityClient"></param>
        /// <param name="identityConfigs"></param>
        /// <param name="mapper"></param>
        /// <param name="systemSettingsRepository"></param>
        /// <param name="systemSettingGroupRepository"></param>
        /// <param name="telRepository"></param>
        /// <param name="systemMenuRepository"></param>
        /// <param name="systemButtonRepository"></param>
        /// <param name="systemAuthorityRepository"></param>
        /// <param name="systemDataAuthorityRepository"></param>
        public SysController(
            ISessionContext sessionContext,
            IUserRepository userRepository,
            IUserCacheManager userCacheManager,
            IIdentityClient identityClient,
            IOptionsSnapshot<IdentityConfigs> identityConfigs,
            IMapper mapper,
            ISystemSettingRepository systemSettingsRepository,
            ISystemSettingGroupRepository systemSettingGroupRepository,
            ITelRepository telRepository,
            ISystemMenuRepository systemMenuRepository,
            ISystemButtonRepository systemButtonRepository,
            ISystemAuthorityRepository systemAuthorityRepository,
            ISystemDataAuthorityRepository systemDataAuthorityRepository)
        {
            _sessionContext = sessionContext;
            _userRepository = userRepository;
            _userCacheManager = userCacheManager;
            _identityClient = identityClient;
            _identityConfigs = identityConfigs;
            _mapper = mapper;
            _systemSettingsRepository = systemSettingsRepository;
            _systemSettingGroupRepository = systemSettingGroupRepository;
            _telRepository = telRepository;
            _systemMenuRepository = systemMenuRepository;
            _systemButtonRepository = systemButtonRepository;
            _systemAuthorityRepository = systemAuthorityRepository;
            _systemDataAuthorityRepository = systemDataAuthorityRepository;
        }

        #region private

        private async Task<bool> IsAccountLock(string userId)
        {
            var response = await _identityClient.IsAccountLockAsync(userId, HttpContext.RequestAborted);
            if (response is null || !response.IsSuccess)
                throw new UserFriendlyException("identity service request fail: IsAccountLockAsync");
            return response.Result;
        }


        private void CheckHttpRequestSuccess(ApiResponse response, string msg)
        {
            if (response == null || !response.IsSuccess)
                throw new UserFriendlyException($"identity service request failed: {msg}");
        }

        #endregion

        #region 系统参数

        /// <summary>
        /// 获取系统参数列表
        /// </summary>
        /// <returns></returns>
        [Permission(EPermission.GetSysSettingsAsync)]
        [HttpGet("getsyssettings")]
        public async Task<List<SystemSettingGroup>> GetSysSettingsAsync()
        {
            return await _systemSettingGroupRepository.QueryExtAsync(x => true, x => x.Includes(d => d.SystemSettings));
        }

        /// <summary>
        /// 更新配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [Permission(EPermission.ModifySettings)]
        [HttpPost("modifysettings")]
        public async Task ModifySettings([FromBody] ModifySettingsRequest request)
        {
            for (int i = 0; i < request.list.Count; i++)
            {
                var model = await _systemSettingsRepository.GetAsync(x => x.Id == request.list[i].id, HttpContext.RequestAborted);
                if (model != null)
                {
                    model.SettingValue = request.list[i].value;
                    await _systemSettingsRepository.UpdateAsync(model, HttpContext.RequestAborted);
                }
            }
        }


        #endregion

        #region 菜单管理
        /// <summary>
        /// 获取菜单
        /// </summary>
        /// <returns></returns>
        //[Permission(EPermission.GetMenuJson)]
        [AllowAnonymous]
        [HttpGet("getmenujson")]
        public async Task<IReadOnlyList<SystemMenu>> GetMenuJson()
        {
            //获取菜单树形
            return await _systemMenuRepository.GetSystemMenus();
        }

        /// <summary>
        /// 新增菜单
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddMenu)]
        [HttpPost("add-menu")]
        public async Task AddMenu([FromBody] AddMenuDto dto)
        {
            var menu = _mapper.Map<SystemMenu>(dto);
            await _systemMenuRepository.AddAsync(menu);
        }

        /// <summary>
        /// 修改菜单
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.UpdateMenu)]
        [HttpPost("update-menu")]
        public async Task UpdateMenu([FromBody] UpdateMenuDto dto)
        {
            var menu = await _systemMenuRepository.GetAsync(dto.Id, HttpContext.RequestAborted);
            if (menu is null)
                throw UserFriendlyException.SameMessage("无效菜单");

            _mapper.Map(dto, menu);
            await _systemMenuRepository.UpdateAsync(menu, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 删除菜单
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Permission(EPermission.RemoveMenu)]
        [HttpDelete("removemenu/{id}")]
        public async Task RemoveMenu(string id)
        {
            var menu = await _systemMenuRepository.GetAsync(id, HttpContext.RequestAborted);
            if (menu is null)
                throw UserFriendlyException.SameMessage("无效菜单");
            //检查是否存在按钮
            var ishave = await _systemButtonRepository.AnyAsync(x => x.MenuId == id);
            if (ishave)
                throw UserFriendlyException.SameMessage("存在按钮权限，暂时不能删除");

            await _systemMenuRepository.RemoveAsync(id, true, HttpContext.RequestAborted);
        }

        #region 按钮管理

        /// <summary>
        /// 通过菜单ID获取按钮列表
        /// </summary>
        /// <param name="menuid"></param>
        /// <returns></returns>
        [Permission(EPermission.GetButtonByMenu)]
        [HttpGet("getbuttonmenu")]
        public async Task<IReadOnlyList<SystemButton>> GetButtonByMenu(string menuid)
        {
            return await _systemButtonRepository.QueryAsync(x => x.MenuId == menuid);
        }

        /// <summary>
        /// 新增按钮
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddButton)]
        [HttpPost("add-button")]
        public async Task AddButton([FromBody] AddButtonDto dto)
        {
            var button = _mapper.Map<SystemButton>(dto);
            await _systemButtonRepository.AddAsync(button);
        }

        /// <summary>
        /// 编辑按钮
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.UpdateButton)]
        [HttpPost("update-button")]
        public async Task UpdateButton([FromBody] UpdateButtonDto dto)
        {
            var button = await _systemButtonRepository.GetAsync(dto.Id, HttpContext.RequestAborted);
            if (button is null)
                throw UserFriendlyException.SameMessage("无效按钮");
            _mapper.Map(dto, button);
            await _systemButtonRepository.UpdateAsync(button);
        }

        /// <summary>
        /// 删除按钮
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Permission(EPermission.RemoveButton)]
        [HttpDelete("removebutton/{id}")]
        public async Task RemoveButton(string id)
        {
            var button = await _systemButtonRepository.GetAsync(id, HttpContext.RequestAborted);
            if (button is null)
                throw UserFriendlyException.SameMessage("无效按钮");
            await _systemButtonRepository.RemoveAsync(id, true, HttpContext.RequestAborted);
        }

        #endregion
        #endregion
    }
}
