﻿using Hotline.Caches;
using Hotline.CallCenter.Calls;
using Hotline.CallCenter.Devices;
using Hotline.CallCenter.Tels;
using Hotline.Permissions;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Enums;
using Hotline.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Exceptions;
using XF.Utility.EnumExtensions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 话机相关接口
    /// </summary>
    public class TelController : BaseController
    {
        private readonly ITelDomainService _telDomainService;
        private readonly IUserCacheManager _userCacheManager;
        private readonly ITypedCache<Tel> _cacheTel;
        private readonly ITypedCache<TelGroup> _cacheTelGroup;
        private readonly ITelCacheManager _telCacheManager;
        private readonly IDeviceManager _deviceManager;
        private readonly IMapper _mapper;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallRepository _callRepository;
        private readonly ISessionContext _sessionContext;

        /// <summary>
        /// 构造
        /// </summary>
        /// <param name="telDomainService"></param>
        /// <param name="userCacheManager"></param>
        /// <param name="cacheTel"></param>
        /// <param name="cacheTelGroup"></param>
        /// <param name="telCacheManager"></param>
        /// <param name="deviceManager"></param>
        /// <param name="mapper"></param>
        /// <param name="callDetailRepository"></param>
        /// <param name="callRepository"></param>
        /// <param name="sessionContext"></param>
        public TelController(
            ITelDomainService telDomainService,
            IUserCacheManager userCacheManager,
            ITypedCache<Tel> cacheTel,
            ITypedCache<TelGroup> cacheTelGroup,
            ITelCacheManager telCacheManager,
            IDeviceManager deviceManager,
            IMapper mapper,
            ICallDetailRepository callDetailRepository,
            ICallRepository callRepository,
            ISessionContext sessionContext)
        {
            _telDomainService = telDomainService;
            _userCacheManager = userCacheManager;
            _cacheTel = cacheTel;
            _cacheTelGroup = cacheTelGroup;
            _telCacheManager = telCacheManager;
            _deviceManager = deviceManager;
            _mapper = mapper;
            _callDetailRepository = callDetailRepository;
            _callRepository = callRepository;
            _sessionContext = sessionContext;
        }
        

        /// <summary>
        /// 查询所有分机（高频）
        /// </summary>
        /// <returns></returns>
        [HttpGet("tels-frequency")]
        public async Task<IReadOnlyList<TelDto>> QueryTelsFrequency()
        {
            var tels = _telCacheManager.GetTels();
            var dtos = new List<TelDto>();
            foreach (var tel in tels)
            {
                var telModel = _mapper.Map<TelDto>(_telCacheManager.GetTel(tel.No));
                var telDto = await _deviceManager.QueryTelAsync(tel.No, HttpContext.RequestAborted);

                if (telDto!=null)
                {
                    telModel.CPN = telDto.CPN;
                    telModel.CDPN = telDto.CDPN;
                    telModel.TelStatusInfo = telDto.TelStatusInfo;
                    telModel.ConversationId = telDto.ConversationId;
                    if (telModel.TelStatus == Share.Enums.ETelStatus.Active)
                        telModel.TelStatus = telDto.TelStatus;

                    if (telModel.TelStatusInfo == ETelStatusInfo.Out)
                    {
                        var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection == ECallDirection.Out && x.ConversationId == telModel.ConversationId);
                        telModel.CallId = temp?.Id;
                    }
                    else if (telModel.TelStatusInfo == ETelStatusInfo.Into)
                    {
                        var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection == ECallDirection.In && x.ConversationId == telModel.ConversationId);
                        telModel.CallId = temp?.Id;
                    }
                }
                dtos.Add(telModel);
            }
            return dtos;
        }

        /// <summary>
        /// 查询当前用户分机（高频）
        /// </summary>
        /// <returns></returns>
        [HttpGet("tel-frequency")]
        public async Task<TelDto>  QueryTelByToken()
        {
            var userid = _sessionContext.RequiredUserId;
            var work = _userCacheManager.GetWorkByUser(userid);
            var telModel = _mapper.Map<TelDto>(_telCacheManager.GetTel(work.TelNo));
            var telDto = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);

            if (telDto!=null)
            {
                telModel.CPN = telDto.CPN;
                telModel.CDPN = telDto.CDPN;
                telModel.TelStatusInfo = telDto.TelStatusInfo;
                telModel.ConversationId = telDto.ConversationId;
                if (telModel.TelStatus == Share.Enums.ETelStatus.Active)
                    telModel.TelStatus = telDto.TelStatus;

                if(telModel.TelStatusInfo== ETelStatusInfo.Out)
                {
                    var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection== ECallDirection.Out && x.ConversationId == telModel.ConversationId);
                    telModel.CallId = temp?.Id;
                }
                else if(telModel.TelStatusInfo == ETelStatusInfo.Into)
                {
                    var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.CallDirection == ECallDirection.In && x.ConversationId == telModel.ConversationId);
                    telModel.CallId = temp?.Id;
                }
            }
            return telModel;
        }


    }
}
